@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2015 Enrico M. Crisostomo
+ * Copyright (c) 2014-2021 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -13,18 +13,15 @@
  * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifdef HAVE_CONFIG_H
-#  include "libfswatch_config.h"
-#endif
-
-#include "gettext_defs.h"
-#include "poll_monitor.hpp"
+#include "libfswatch/gettext_defs.h"
 #include <unistd.h>
 #include <cstdlib>
 #include <fcntl.h>
 #include <iostream>
 #include <utility>
-#include "c/libfswatch_log.h"
+#include "libfswatch/libfswatch_config.h"
+#include "libfswatch/c/libfswatch_log.h"
+#include "poll_monitor.hpp"
 #include "path_utils.hpp"
 #include "libfswatch_map.hpp"
 
@@ -34,6 +31,8 @@
 #elif defined HAVE_STRUCT_STAT_ST_MTIMESPEC
 #  define FSW_MTIME(stat) (stat.st_mtimespec.tv_sec)
 #  define FSW_CTIME(stat) (stat.st_ctimespec.tv_sec)
+#else
+#  error "Either HAVE_STRUCT_STAT_ST_MTIME or HAVE_STRUCT_STAT_ST_MTIMESPEC must be defined"
 #endif
 
 namespace fsw
@@ -41,29 +40,22 @@ namespace fsw
   using std::vector;
   using std::string;
 
-  typedef struct poll_monitor::poll_monitor_data
+  using poll_monitor_data = struct poll_monitor::poll_monitor_data
   {
     fsw_hash_map<string, poll_monitor::watched_file_info> tracked_files;
-  }
-  poll_monitor_data;
-
-  REGISTER_MONITOR_IMPL(poll_monitor, poll_monitor_type);
+  };
 
   poll_monitor::poll_monitor(vector<string> paths,
                              FSW_EVENT_CALLBACK *callback,
                              void *context) :
     monitor(std::move(paths), callback, context)
   {
-    previous_data = new poll_monitor_data();
-    new_data = new poll_monitor_data();
+    previous_data.reset(new poll_monitor_data());
+    new_data.reset(new poll_monitor_data());
     time(&curr_time);
   }
 
-  poll_monitor::~poll_monitor()
-  {
-    delete previous_data;
-    delete new_data;
-  }
+  poll_monitor::~poll_monitor() = default;
 
   bool poll_monitor::initial_scan_callback(const string& path,
                                            const struct stat& stat)
@@ -158,7 +150,7 @@ namespace fsw
     vector<fsw_event_flag> flags;
     flags.push_back(fsw_event_flag::Removed);
 
-    for (auto& removed : previous_data->tracked_files)
+    for (const auto& removed : previous_data->tracked_files)
     {
       events.emplace_back(removed.first, curr_time, flags);
     }
@@ -166,16 +158,15 @@ namespace fsw
 
   void poll_monitor::swap_data_containers()
   {
-    delete previous_data;
-    previous_data = new_data;
-    new_data = new poll_monitor_data();
+    previous_data = std::move(new_data);
+    new_data.reset(new poll_monitor_data());
   }
 
   void poll_monitor::collect_data()
   {
     poll_monitor_scan_callback fn = &poll_monitor::intermediate_scan_callback;
 
-    for (string& path : paths)
+    for (const string& path : paths)
     {
       scan(path, fn);
     }
@@ -188,7 +179,7 @@ namespace fsw
   {
     poll_monitor_scan_callback fn = &poll_monitor::initial_scan_callback;
 
-    for (string& path : paths)
+    for (const string& path : paths)
     {
       scan(path, fn);
     }
