@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2021 Enrico M. Crisostomo
+ * Copyright (c) 2014-2022 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -16,10 +16,10 @@
 /**
  * @file
  * @brief Main `libfswatch` source file.
- * @copyright Copyright (c) 2014-2016 Enrico M. Crisostomo
+ * @copyright Copyright (c) 2014-2022 Enrico M. Crisostomo
  * @license GNU General Public License v. 3.0
  * @author Enrico M. Crisostomo
- * @version 1.10.0
+ * @version 1.17.0
  */
 /**
  * @mainpage
@@ -275,6 +275,20 @@
 /**
  * @page history History
  *
+ * @section v1200 12:0:0
+ *
+ *   - Implement event bubbling (see the monitor class) by grouping events using
+ *     the (time, path) tuple, to emit a single event with the union of all the
+ *     flags.
+ *
+ *   - Refactor the fsevents_monitor class to replace usages of the deprecated
+ *     function FSEventStreamScheduleWithRunLoop and substituting it with
+ *     FSEventStreamSetDispatchQueue and an implementation based on dispatch
+ *     queues.
+ *
+ *   - Improve the responsiveness of the fsevents_monitor on macOS by adding
+ *     support for the kFSEventStreamCreateFlagNoDefer flag.
+ *
  * @section v1110 11:1:0
  *
  *   - Fix monitor_factory::create_monitor ignoring the monitor type and always
@@ -418,12 +432,7 @@
  *
  *   - The order in the filter definition has no effect.
  */
-
-#ifdef HAVE_CONFIG_H
-#  include "libfswatch_config.h"
-#endif
-
-#include "gettext_defs.h"
+#include "libfswatch/gettext_defs.h"
 #include <iostream>
 #include <ctime>
 #include <cstdlib>
@@ -431,17 +440,18 @@
 #include <memory>
 #include <vector>
 #include <map>
+#include "libfswatch/libfswatch_config.h"
 #include "libfswatch.h"
-#include "../c++/libfswatch_map.hpp"
-#include "../c++/filter.hpp"
-#include "../c++/monitor.hpp"
-#include "../c++/monitor_factory.hpp"
-#include "../c++/libfswatch_exception.hpp"
+#include "libfswatch/c++/libfswatch_map.hpp"
+#include "libfswatch/c++/filter.hpp"
+#include "libfswatch/c++/monitor.hpp"
+#include "libfswatch/c++/monitor_factory.hpp"
+#include "libfswatch/c++/libfswatch_exception.hpp"
 
 using namespace std;
 using namespace fsw;
 
-typedef struct FSW_SESSION
+using FSW_SESSION = struct FSW_SESSION
 {
   vector<string> paths;
   fsw_monitor_type type;
@@ -456,7 +466,7 @@ typedef struct FSW_SESSION
   vector<fsw_event_type_filter> event_type_filters;
   map<string, string> properties;
   void *data;
-} FSW_SESSION;
+};
 
 static bool fsw_libfswatch_verbose = false;
 static FSW_THREAD_LOCAL FSW_STATUS last_error;
@@ -481,12 +491,12 @@ FSW_STATUS fsw_init_library()
   return FSW_OK;
 }
 
-typedef struct fsw_callback_context
+using fsw_callback_context = struct fsw_callback_context
 {
   FSW_HANDLE handle;
   FSW_CEVENT_CALLBACK callback;
   void *data;
-} fsw_callback_context;
+};
 
 void libfsw_cpp_callback_proxy(const std::vector<event>& events,
                                void *context_ptr)
@@ -497,7 +507,7 @@ void libfsw_cpp_callback_proxy(const std::vector<event>& events,
 
   const fsw_callback_context *context = static_cast<fsw_callback_context *> (context_ptr);
 
-  fsw_cevent *const cevents = static_cast<fsw_cevent *> (malloc(
+  auto *const cevents = static_cast<fsw_cevent *> (malloc(
     sizeof(fsw_cevent) * events.size()));
 
   if (cevents == nullptr)
@@ -555,7 +565,7 @@ void libfsw_cpp_callback_proxy(const std::vector<event>& events,
 
 FSW_HANDLE fsw_init_session(const fsw_monitor_type type)
 {
-  FSW_SESSION *session = new FSW_SESSION{};
+  auto *session = new FSW_SESSION{};
   session->type = type;
 
   return session;
@@ -574,10 +584,10 @@ int create_monitor(const FSW_HANDLE handle, const fsw_monitor_type type)
     if (session->monitor)
       return fsw_set_last_error(int(FSW_ERR_MONITOR_ALREADY_EXISTS));
 
-    if (!session->paths.size())
+    if (session->paths.empty())
       return fsw_set_last_error(int(FSW_ERR_PATHS_NOT_SET));
 
-    fsw_callback_context *context_ptr = new fsw_callback_context;
+    auto *context_ptr = new fsw_callback_context{};
     context_ptr->handle = session;
     context_ptr->callback = session->callback;
     context_ptr->data = session->data;
@@ -588,7 +598,7 @@ int create_monitor(const FSW_HANDLE handle, const fsw_monitor_type type)
                                                                context_ptr);
     session->monitor = current_monitor;
   }
-  catch (libfsw_exception& ex)
+  catch (const libfsw_exception& ex)
   {
     return fsw_set_last_error(int(ex));
   }
@@ -606,7 +616,7 @@ FSW_STATUS fsw_add_path(const FSW_HANDLE handle, const char *path)
     return fsw_set_last_error(int(FSW_ERR_INVALID_PATH));
 
   FSW_SESSION *session = get_session(handle);
-  session->paths.push_back(path);
+  session->paths.emplace_back(path);
 
   return fsw_set_last_error(FSW_OK);
 }
@@ -743,7 +753,7 @@ FSW_STATUS fsw_start_monitor(const FSW_HANDLE handle)
 
     session->monitor->start();
   }
-  catch (libfsw_exception& ex)
+  catch (const libfsw_exception& ex)
   {
     return fsw_set_last_error(int(ex));
   }
@@ -769,7 +779,7 @@ FSW_STATUS fsw_stop_monitor(const FSW_HANDLE handle)
 
     session->monitor->stop();
   }
-  catch (libfsw_exception& ex)
+  catch (const libfsw_exception& ex)
   {
     return fsw_set_last_error(int(ex));
   }
