@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015 Enrico M. Crisostomo
+ * Copyright (c) 2015-2021 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -28,23 +28,28 @@
 #ifndef LIBFSWATCH_TYPES_H
 #define LIBFSWATCH_TYPES_H
 
+#include "libfswatch/libfswatch_config.h"
+
 #ifdef __cplusplus
 extern "C"
 {
 #endif
 
+/**
+ * @brief Opaque type representing a monitoring session.
+ */
+struct FSW_SESSION;
+
 /**
  * @brief Handle to a monitoring session.
  */
-typedef unsigned int FSW_HANDLE;
+typedef struct FSW_SESSION *FSW_HANDLE;
 
 /**
  * @brief Status of a library call.
  */
 typedef int FSW_STATUS;
 
-#define FSW_INVALID_HANDLE -1
-
 #if defined(HAVE_CXX_THREAD_LOCAL)
 # define FSW_THREAD_LOCAL thread_local
 #else
